clear all

dynare simpless
%load simpless_results;

% Restricted parameters
restr_names = []; 
restr_indx = [];
nrestr = size(restr_names,1);
for i=1:nrestr
    restr_indx = [restr_indx; strmatch(restr_names(i,:),M_.param_names,'exact')];
end
restr_values = M_.params(restr_indx);

% Parameter values
x = zeros(M_.param_nbr,1);
for i=1:M_.param_nbr
    x(i) = M_.params(i);
end

% Names and values of unrestricted parameters
theta_names = cellstr(M_.param_names);
theta_names(restr_indx,:)=[];
x(restr_indx,:)=[];

% Number of unrestricted parameters
ntheta = size(x,1);

% Bounds for unrestricted parameters
theta_bounds = kron(ones(ntheta,1),[-inf inf]);
theta_bounds = [0 1; 0 1];

% Collecting info about parameters
params_.theta_names     = theta_names;
params_.ntheta          = ntheta;
params_.restr_names     = restr_names;
params_.restr_values    = restr_values;
params_.nrestr          = nrestr;
params_.theta_bounds    = theta_bounds;



[q n] = size(oo_.dr.ghx); %n is number of states
q = q-n; %number of non-state variables
k = 1; %number of observables (equal to number of shocks)
h = (k-1)*k/2; %number of free elements in orthogonal matrix
model_.n     = n;
model_.q     = q;
model_.k     = k;
model_.h     = h;

% Model solution
A = oo_.dr.ghx(1:n,1:n);
B = oo_.dr.ghu(1:n,1:k);
F1 = oo_.dr.ghx(n+1:end,:);
G1 = oo_.dr.ghu(n+1:end,1:k);

H=[1 1 0];
Hs= H(:,1:n);
Hp= H(:,n+1:end);

C = Hs*A+Hp*F1;
D = Hs*B+Hp*G1;

% Collecting info about model
model_.A     = A;
model_.B     = B;
model_.C     = C;
model_.D     = D;
model_.F1    = F1;
model_.G1    = G1;
model_.Hs    = Hs;
model_.Hp    = Hp;

% Vectorizing F matrix (start from F1)
xF= reshape(F1,q*n,1);

% Vectorizing G matrix (start from G1)
xG= reshape(G1,q*k,1);

% Vectorizing T matrix (start from I)
T = eye(n);
xT= reshape(T,n*n,1);

% Free elements in H matrix (start from I)
xH= zeros(h,1);

% Original model solution
xorig = [x; xF; xG; xT; xH];


% Initialization
options = optimset('TolFun',1e-20,'TolX',1e-20,'Algorithm','trust-region-reflective','Display','off');
count = 0;


% Parametrizing heat map
n_kk = 10; %number of replications for each point (runs for 2-3 days for 10000 used in the paper)
par = 4; % number of cores for parallel computing (1: no parallelization)
n_kk = ceil(n_kk/par);
parpool(par);
n_ij = 10; %number of grid points per dimension
eps = 1/n_ij/100;
rescale = 1; %scaling factor for randomizing distribution (nu_i)

% Initialization
heatmap_points = zeros(n_ij+1,n_ij+1);


% Main loop checking identification at each node of the grid point
tic;
parfor xx=1:par 
  
heatmap_points = [];
for ii = 0:n_ij
    for jj = 0:n_ij
        disp(['worker ' num2str(xx) ': node [' num2str(ii) ' ' num2str(jj) ']']);
        thet_sim = zeros(1,n_kk);
        for kk = 1:n_kk
            exitflag=0;
            while exitflag<1
                x0 = xorig;
                x0(1:2) = [ii/n_ij; jj/n_ij];
                if ii == 0; x0(1) = [ii/n_ij+eps]; end;
                if ii == n_ij; x0(1) = [ii/n_ij-eps]; end;
                if jj == 0; x0(2) = [jj/n_ij+eps]; end;
                if jj == n_ij; x0(2) = [jj/n_ij-eps]; end;
                x0(3:end) = xorig(3:end) + randn(size(xorig(3:end),1),1)*rescale;
                fsys9 = @(x)sys9(x,model_,params_);
                [x fval exitflag output] = fsolve(fsys9,x0,options); 
            end
            thet_sim(kk) = ~any(abs(xorig(1:ntheta)-x(1:ntheta))>1e-5);
        end
        heatmap_points(jj+1,ii+1) = mean(thet_sim);
    end
end

heatmap_all(:,:,xx) = heatmap_points;

end
delete(gcp)
toc;


% Generating heatmap figure (one panel in Figure 1 from the paper)
heatmap_all = 1 - heatmap_all(:,:,1:par);
heatmap_points = mean(heatmap_all,3);
figure;
contourf(0.001:(0.1):1.001,0.001:(0.1):1.001,heatmap_points,[0.02:0.02:0.4],'ShowText','on')
set(gca,'YDir','normal')
xlabel('$\bar{\alpha_1}^0$','Interpreter','latex')
ylabel('$\bar{\alpha_2}^0$','Interpreter','latex')
colormap(hot(8))
colormap(flipud(colormap))

